/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2021, Josef Hahn

 This file is part of Yaji.

 Yaji is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Yaji is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Yaji.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


class MyWidget extends clove.NameScope(clove.Grid) {

    constructor(config, domnode) {
        super(clove.utils.applyDefaultsToConfig(config, {
            children: [
            {label: '2^10', row: 1, col: 0, style: 'font-size:1.4em'},
            {label: '1/0', row: 2, col: 0, style: 'font-size:1.4em'},
            {label: 'classic', row: 0, col: 1, style: 'font-size:1.4em'},
            {view: 'Spacer', row: 0, col: 2, width: '5pt'},
            {label: 'async', row: 0, col: 3, style: 'font-size:1.4em'},

            {view: 'Label', row: 1, col: 1, name: 'classic_value'},
            {view: 'Label', row: 1, col: 3, name: 'async_value'},
            {view: 'Label', row: 2, col: 1, name: 'classic_witherror'},
            {view: 'Label', row: 2, col: 3, name: 'async_witherror'},]
        }), domnode);
    }

    doinit() {
        super.doinit();
        this._ajax_classic();
        this._ajax_async();
        this._ajax_classic_witherror();
        this._ajax_async_witherror();
    }

    _ajax_classic() {  // classic way
        var self = this;
        yaji.ajax({
            url: 'mul',
            data: {x: 32, y: 32},
            success: function(r1) {
                self.getByName("classic_value").setLabel(r1);
            }
        });
    }

    _ajax_classic_witherror() {  // classic way with error
        var self = this;
        yaji.ajax({
            url: 'div',
            data: {x: 1, y: 0},
            error: function() {
                self.getByName("classic_witherror").setLabel("boom!");
            }
        });
    }

    async _ajax_async() {  // async way
        var x1 = yaji.ajax({url: 'mul', data: {x: 2, y: 2}});
        var x2 = yaji.ajax({url: 'mul', data: {x: 2, y: 2}});
        var x3 = yaji.ajax({url: 'mul', data: {x: await x1, y: await x2}});
        var x4 = yaji.ajax({url: 'mul', data: {x: await x3, y: 4}});
        var x5 = yaji.ajax({url: 'mul', data: {x: await x3, y: await x4}});
        this.getByName("async_value").setLabel(await x5);
    }

    async _ajax_async_witherror() {  // async way with error
        try {
            await yaji.ajax({url: 'div', data: {x: 1, y: 0}});
        }
        catch (e) {
            this.getByName("async_witherror").setLabel("boom!");
        }
    }

}
