#!/usr/bin/python3

# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of Yaji.
#
# Yaji is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yaji is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Yaji.  If not, see <http://www.gnu.org/licenses/>.

import ___sampleapp  # this is just an internal hack - just ignore it and don't copy it!

import datetime
import threading
import time

import yaji


class Application(yaji.Application):

    def __init__(self):
        super().__init__(head1="Yaji Sample", head2="clock", stop_implicitly_when_browser_closed=True)
        self.__clock = self.create_datastore()
        self.setbodyleft(yaji.VerticalStack(
            yaji.Spacer(vstretch=1),
            yaji.View('Label', label=self.binddata(self.__clock),
                      style='font-size:5em;letter-spacing:3pt;text-align:center;', hstretch=1),
            yaji.Spacer(vstretch=1),
            hstretch=1))

    def start(self):
        super().start()
        threading.Thread(target=self._clockthreadtarget, daemon=True).start()

    def _clockthreadtarget(self):
        while self.isrunning:
            self.__clock.setvalue(datetime.datetime.now().strftime("%H:%M:%S"))
            time.sleep(1)


if __name__ == '__main__':
    Application().start()
