# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of Yaji.
#
# Yaji is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yaji is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Yaji.  If not, see <http://www.gnu.org/licenses/>.

import base64

from .core import Middleware


class AuthMiddleware(Middleware):
    """
    Middleware for authentication.
    """

    def __init__(self, *, realm: str):
        """
        :param realm: The http auth realm name.
        """
        super().__init__()
        self.__realm = realm

    def _authenticate_by_password(self, username: str, password: str) -> bool:
        """
        This method authenticates a user by a password.
        |dooverride|

        :return: `True` for successful authentication.
        """
        return False

    @Middleware.request_processor(index=-1_000_000)
    def _checkauth(self, request):
        authorized = False
        hauth = request.header.get('authorization', None)
        if hauth and hauth.lower().startswith("basic "):
            hsauth = base64.b64decode(hauth[6:].encode()).decode()
            icln = hsauth.find(":")
            if icln > -1:
                authorized = self._authenticate_by_password(hsauth[:icln], hsauth[icln+1:])
        if not authorized:
            request.set_response_header('WWW-Authenticate', f'Basic realm="{self.__realm}"')
            request.response_httpcode = 401
            request.response_body = None
            request.response_errortext = None
            request.skip_processing = True
