# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of Yaji.
#
# Yaji is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yaji is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Yaji.  If not, see <http://www.gnu.org/licenses/>.

from typing import *


class TrStr:
    """
    An i18n-aware string for displaying on browser side. When returned in a json structure, the browser side
    transparently gets a translated version.
    """

    def __init__(self, stringname: str, **args: Any):
        """
        :param stringname: The string name (as used in clove.i18n.addString() on browser side).
        :param args: String pieces for replacing :samp:`"{foo}"` patterns in the translation with.
        """
        self.__stringname = stringname
        self.__args = args

    @property
    def stringname(self) -> str:
        """
        The string name (as used for referencing the translations for one text).
        """
        return self.__stringname

    @property
    def args(self) -> Dict[str, Any]:
        """
        String pieces for replacing :samp:`"{foo}"` patterns in the translation with.
        """
        return self.__args

    def _to_simple_repr_(self):
        return 'TrStr', {'stringname': self.stringname, 'args': self.args}


#: Type annotation for something that can be either a :samp:`str` or a :py:class:`TrStr`.
TrStrOrStrTyping = Union[str, TrStr]
