# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

"""
Dependencies from the Python world.
"""

from typing import *

import anise.framework.features

dependencies = anise.framework.features.loadfeature("dependencies").featuremodule


class PythonDependency(dependencies.Dependency):
    """
    A dependency describing the required Python interpreter.
    """

    # noinspection PyShadowingBuiltins
    def __init__(self, type: 'dependencies.Type', version: str, **kwargs):
        """
        :param type: The dependency type.
        :param version: The version of the required Python interpreter.
        :param kwargs: Additional dependency infos.
        """
        def setdefault(k, v):
            if k not in kwargs:
                kwargs[k] = v
        setdefault("objectname", f"python-{version}")
        setdefault("displayname", f"Python {version}")
        setdefault("icon", "python")
        postfix = "" if version[0] == "2" else version[0]
        setdefault("debian", [f"python{postfix} (>= {version})"])
        super().__init__(type, **kwargs)


class DjangoDependency(dependencies.Dependency):
    """
    A dependency describing the required Django framework installation.
    """

    # noinspection PyShadowingBuiltins
    def __init__(self, type: 'dependencies.Type', version: str, **kwargs):
        """
        :param type: The dependency type.
        :param version: The version of the required Django framework.
        :param kwargs: Additional dependency infos.
        """
        def setdefault(k, v):
            if k not in kwargs:
                kwargs[k] = v
        setdefault("objectname", f"django-{version}")
        setdefault("displayname", f"Django {version}")
        setdefault("icon", "django")
        setdefault("debian", [f"python3-django (>= {version})"])
        super().__init__(type, **kwargs)


if TYPE_CHECKING:
    import anise.features.dependencies as dependencies
