# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.framework.features
import anise.framework.projects

projectdescriptioneditor = anise.framework.features.loadfeature("projectdescriptioneditor").featuremodule
projectdescriptioneditorfiletransfer = \
    anise.framework.features.loadfeature("projectdescriptioneditor.filetransfer").featuremodule


# noinspection PyUnusedLocal
@anise.framework.features.hookhandler(projectdescriptioneditor.HOOK_GET_FEATURE_ACTIONS)
class ProjectDescriptionEditorAssistant(projectdescriptioneditor.ProjectDescriptionEditorAssistant):

    def __init__(self):
        projectdescriptioneditor.ProjectDescriptionEditorAssistant.__init__(
            self, "Manage homepage destinations", "Specify a destination for deployment of the project homepage at "
                                                  "release time.", "homepage")

    def getcustomactions(self, ais):
        return [projectdescriptioneditor.CustomAction("Add new homepage destination", self._add_homepage_destination,
                                                      icon="userfeedback/add.png")]

    def _add_homepage_destination(self, ais):
        ft = projectdescriptioneditorfiletransfer.ask_new_filetransfer("Please choose the homepage deployment mode.")
        if ft is not None:
            reltask = anise.framework.projects.IntermediateStructure.Let(
                k="releasing.tasks.add",
                source=anise.framework.projects.IntermediateStructure.Value(
                    "files.TaskExecution", task=anise.framework.projects.IntermediateStructure.Value(
                        "objectref", oref="homepage.makehomepage")))
            for eft in ft.args():
                reltask.setarg(eft, ft.getarg(eft))
            ais.addentry(reltask)

    def getentries(self, ais):
        r = []
        for e in ais.entries():
            if isinstance(e, anise.framework.projects.IntermediateStructure.Let) and e.key == "releasing.tasks.add":
                te = e.getarg("source")
                if te is not None and te.vtype == "files.TaskExecution":
                    tsk = te.getarg("task")
                    if tsk is not None:
                        if tsk.oref == "homepage.makehomepage":
                            r.append(e)
        return r

    def getentrylabel(self, ais, entry):
        te = entry.getarg("destination")
        if te is not None:
            return str(te)
