# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.framework.features
import anise.framework.projects

from anise.framework.projects import universe

projectdescriptioneditor = anise.framework.features.loadfeature("projectdescriptioneditor").featuremodule


# noinspection PyUnusedLocal
@anise.framework.features.hookhandler(projectdescriptioneditor.HOOK_GET_FEATURE_ACTIONS)
class ProjectDescriptionEditorAssistant(projectdescriptioneditor.ProjectDescriptionEditorAssistant):

    def __init__(self):
        projectdescriptioneditor.ProjectDescriptionEditorAssistant.__init__(
            self, "Manage media galleries", "Specify media galleries, for example including screenshots. They will be "
                                            "used in some content generators, like for the project homepage.",
            "mediagalleries")

    def getcustomactions(self, ais):
        return [projectdescriptioneditor.CustomAction("Add new media gallery", self._add_imagegallery,
                                                      icon="userfeedback/add.png")]

    def _add_imagegallery(self, ais):
        name = universe.ui.userfeedback.input("Please enter a name for the media gallery.", "Screenshots")
        if name is None:
            return
        gallerydir = universe.ui.userfeedback.input(
            "Please the path to the media gallery files (relative to the project root directory).",
            "_meta/"+name.lower())
        if gallerydir is None:
            return
        newlet = anise.framework.projects.IntermediateStructure.Let(
            k="mediagalleries.pool.add", name=anise.framework.projects.IntermediateStructure.Value.fromstring(name))
        newlet.setarg("gallery", anise.framework.projects.IntermediateStructure.Value(
            t="mediagalleries.MediaGallery",
            path=anise.framework.projects.IntermediateStructure.Value.fromstring(gallerydir)))
        ais.addentry(newlet)

    def getentries(self, ais):
        r = []
        for e in ais.entries():
            if isinstance(e, anise.framework.projects.IntermediateStructure.Let) \
                    and e.key == "mediagalleries.pool.add":
                r.append(e)
        return r

    def getentrylabel(self, ais, entry):
        return entry.getarg("name").value
