# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

"""
Feature for generic unit testing.
"""

from typing import *

import anise.framework.features
import anise.utils

testing = anise.framework.features.loadfeature("testing").featuremodule


def generictest(testrun: 'testing.Internals.TestRun', command: List[str]) -> None:
    """
    Runs an external command as test and interpret it as successful, if the exit code is :samp:`0`.

    :param testrun: A test run.
    :param command: The external command (as list of arguments).
    """
    testrun.begintest()
    r, o = anise.utils.basic.call(*command)
    success = testing.Internals.TestRun.SUCCESS_SUCCESSFUL if (r == 0) else testing.Internals.TestRun.SUCCESS_FAILED
    testrun.endtest(o, success)


if TYPE_CHECKING:
    import anise.features.testing as testing
