# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

"""
Feature for helpers related to terminal-based user interfaces.
"""

import sys


def showhead(head: str, msg: str, withterm: bool = True) -> None:
    sys.stderr.originalstream.write(f"\n\033[30;104m\n{head}\n\033[30;107m\n{msg} \033[33;107m")
    if withterm:
        sys.stderr.originalstream.write("\033[0m\n")


def prompt(head: str, msg: str) -> str:
    showhead(head, msg, withterm=False)
    res = sys.stdin.readline()
    sys.stderr.originalstream.write("\033[0m\n")
    if res[-1:] in "\n\r":
        res = res[:-1]
    return res
