# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.framework.exceptions
import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="foo" v="" />
    <val k="bar" v="" />
    <val k="baz" v="" />
    <val k="result" oref="myfeature3.result" />
    <val k="bamm" oref="myfeature3.bamm" />
    <val k="bomm" oref="myfeature3.bomm" />
    <val k="bimm" oref="myfeature3.bimm" />
</aniseprojectdesc>
"""

_myfeature = """
import anise.framework.features

from anise.framework.projects import universe


def result():
    for h in universe.gethooks("FOOHOOK"):
        universe.foo = h(universe.foo)
    for h in universe.gethooks("BARHOOK"):
        universe.bar = h(universe.bar)
    for h in universe.gethooks("BAZHOOK"):
        universe.baz = h(universe.baz)
    return universe.foo + " " + universe.bar + " " + universe.baz
def bamm():
    for h in universe.gethooks("BAMMHOOK"):
        h()
def bomm():
    for h in universe.gethooks("BOMMHOOK"):
        h()
def bimm():
    for h in universe.gethooks("BIMMHOOK"):
        h()

@anise.framework.features.hookhandler("FOOHOOK", provides=["c"], requires=["b"])
def hook1c(foo):
    return "a" + foo

@anise.framework.features.hookhandler("FOOHOOK", provides=["a"])
def hook1a(foo):
    return "L" + foo

@anise.framework.features.hookhandler("FOOHOOK", provides=["e"], requires=["d"])
def hook1e(foo):
    return foo[:-1] + foo[-1].swapcase()

@anise.framework.features.hookhandler("FOOHOOK", provides=["b"], requires=["a"])
def hook1b(foo):
    return foo.swapcase()

@anise.framework.features.hookhandler("FOOHOOK", provides=["d"], requires=["c"])
def hook1d(foo):
    return foo + "L"


@anise.framework.features.hookhandler("BARHOOK", provides=["e"], prepares=["f"], requires=["d"])
def hook2e(bar):
    return bar + "e"

@anise.framework.features.hookhandler("BARHOOK", provides=["f"], requires=["d"])
def hook2f(bar):
    return bar + "m"

@anise.framework.features.hookhandler("BARHOOK", provides=["d"], requires=["c"])
def hook2d(bar):
    return bar + "t"

@anise.framework.features.hookhandler("BARHOOK", provides=["b"], requires=["a"], prepares=["c"])
def hook2b(bar):
    return bar + "y"

@anise.framework.features.hookhandler("BARHOOK", provides=["a", "PRE"])
def hook2a(bar):
    return bar + "s"

@anise.framework.features.hookhandler("BARHOOK", provides=["g", "POST"])
def hook2g(bar):
    return bar + "s"

@anise.framework.features.hookhandler("BARHOOK", provides=["c"], prepares=["d"])
def hook2c(bar):
    return bar + "s"


@anise.framework.features.hookhandler("BAZHOOK", provides=["e", "POST"], requires=["d"])
def hook3e(baz):
    return baz[0:1].swapcase() + baz[1:2]

@anise.framework.features.hookhandler("BAZHOOK", provides=["d", "POST"], prepares=["e"], requires=["c"])
def hook3d(baz):
    return baz.swapcase()

@anise.framework.features.hookhandler("BAZHOOK", provides=["c"], requires=["b"])
def hook3c(baz):
    return baz + "O"

@anise.framework.features.hookhandler("BAZHOOK", provides=["b", "PRE"])
def hook3b(baz):
    return baz + "g"

@anise.framework.features.hookhandler("BAZHOOK", provides=["a", "PRE"], prepares=["b"])
def hook3a(baz):
    return baz.upper()


@anise.framework.features.hookhandler("BAMMHOOK", provides=["a"], requires=["c"])
def hook4a():
    raise Exception("baam1")

@anise.framework.features.hookhandler("BAMMHOOK", provides=["b"], requires=["a"])
def hook4b():
    raise Exception("baam2")

@anise.framework.features.hookhandler("BAMMHOOK", provides=["c"], requires=["b"])
def hook4c():
    raise Exception("baam3")


@anise.framework.features.hookhandler("BOMMHOOK", provides=["a"], prepares=["b"])
def hook5a():
    raise Exception("boom1")

@anise.framework.features.hookhandler("BOMMHOOK", provides=["b"], prepares=["c"])
def hook5b():
    raise Exception("boom2")

@anise.framework.features.hookhandler("BOMMHOOK", provides=["c"], prepares=["a"])
def hook5c():
    raise Exception("boom3")


@anise.framework.features.hookhandler("BIMMHOOK", provides=["a", "PRE"], requires=["b"])
def hook6a():
    raise Exception("biim1")

@anise.framework.features.hookhandler("BIMMHOOK", provides=["b"], requires=["c"])
def hook6b():
    raise Exception("biim2")

@anise.framework.features.hookhandler("BIMMHOOK", provides=["c", "POST"])
def hook6c():
    raise Exception("biim3")
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        super().__init__(*a, projectdesc=_projectdesc, **kwa)

    def test_a(self):
        anise.utils.basic.mkdirs(self.mydir + "/features")
        with open(self.mydir + "/features/myfeature3.py", "w") as f:
            f.write(_myfeature)
        loadfeaturesfrom = ["builtin", self.mydir + "/features"]
        self.assertEqual(self.runanise("result", loadfeaturesfrom=loadfeaturesfrom), "all systems go")

        with self.assertRaises(anise.data.PyDepsEngine.depsengine.CircularDependenciesError):
            self.runanise("bamm", loadfeaturesfrom=loadfeaturesfrom)
        with self.assertRaises(anise.data.PyDepsEngine.depsengine.CircularDependenciesError):
            self.runanise("bomm", loadfeaturesfrom=loadfeaturesfrom)
        with self.assertRaises(anise.data.PyDepsEngine.depsengine.CircularDependenciesError):
            self.runanise("bimm", loadfeaturesfrom=loadfeaturesfrom)
