# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="base.authors"><val v="John Doe" /></val>
    <val k="base.license" oref="licensing.AGPLv3" />
    <let k="base.pyscript">
        <val k="hook" oref="engine.HOOK_AFTER_PREPARATION"/>
        <val k="code">
def fct():
    def bartask():
        x = files.TempDir()
        utils.basic.writetofile(x.path+"/xx", "XX")
        return x
    universe.bartask = bartask

    def result():
        pyunit = universe.pyunit
        filestructure = files.TempDir()
        utils.basic.writetofile(filestructure.path+"/yy", "YY")
        p = files.MergedFilestructure(frommap={
                    '/foo' : universe.packages.RawPackage(),
                    '/bar' : files.TaskExecution(task=universe.bartask),
                    '/baz/a' : filestructure,
                    '/baz/b' : files.MergedFilestructure(fromlist=[
                                                             files.Filestructure("x", relative=True),
                                                             files.Filestructure("y", relative=True)
                                                         ]),
                    '/baz/c' : files.Filestructure(filestructure.path+"/yy"),
                }).path
        l = os.listdir(p + "/foo")
        for x in ["x", "y", "_projectdesc"]:
            pyunit.assertTrue(x in l)
        pyunit.assertEqual("XX", utils.basic.readfromfile(p + "/bar/xx"))
        pyunit.assertEqual("YY", utils.basic.readfromfile(p + "/baz/a/yy"))
        pyunit.assertEqual("YY", utils.basic.readfromfile(p + "/baz/c"))
        pyunit.assertEqual("aa", utils.basic.readfromfile(p + "/baz/b/x"))
        pyunit.assertEqual("bb", utils.basic.readfromfile(p + "/baz/b/y"))
    universe.result = result
        </val>
    </let>
</aniseprojectdesc>
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        super().__init__(*a, projectdesc=_projectdesc, **kwa)

    def test_a(self):
        anise.utils.basic.writetofile(self.mydir + "/x", "aa")
        anise.utils.basic.writetofile(self.mydir + "/y", "bb")
        self.runanise("result", values={'pyunit': self})
