# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.framework.files
import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="name" v="myprojectname" />
    <val k="base.version" v="1.2" />
    <val k="base.summary" v="mysummary" />
    <val k="base.email" v="my@mail.address" />
    <val k="base.authors"><val v="John Doe" /></val>
    <val k="base.license" oref="licensing.PublicDomain" />
    <val k="base.maturity" oref="base.Maturity.ProductionStable" />
    <let k="base.set_basecolor">
        <val k="r" t="float" v="0.1" />
        <val k="g" t="float" v="0.1" />
        <val k="b" t="float" v="0.1" />
    </let>
    <val k="base.longdescription" v="mylongdescription" />
    <let k="releasing.tasks.add" destination="{0}">
        <val k="task" oref="filetransfer.localupload" />
        <val k="source" t="files.TaskExecution">
            <val k="task" oref="homepage.makehomepage" />
        </val>
    </let>
    <let k="dependencies.pool.add">
        <val t="dependencies.Dependency" k="dependency">
            <val oref="dependencies.Type.Recommended_HasAlternatives" k="type" />
            <val v="foodep" k="objectname" /><val v="linux" k="icon" />
        </val>
    </let>
    <let k="dependencies.pool.add">
        <val t="dependencies.Dependency" k="dependency">
            <val oref="dependencies.Type.Optional" k="type" />
            <val v="bardep" k="objectname" /><val v="svn" k="icon" />
        </val>
    </let>
</aniseprojectdesc>
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        self.destination = anise.framework.files.TempDir().path
        super().__init__(*a, projectdesc=_projectdesc.format(self.destination), **kwa)

    def test_a(self):
        self.runanise("releasing.release")
        p = self.destination
        indexhtml = anise.utils.basic.readfromfile(p + "/myprojectname/index.html")
        self.assertTrue("dependencies" in indexhtml)
        self.assertTrue("foodep" in indexhtml)
        self.assertTrue("bardep" in indexhtml)
        self.assertTrue("linux" in indexhtml)
        self.assertTrue("svn" in indexhtml)
        self.assertTrue("optional" in indexhtml)
        self.assertTrue("recommended" in indexhtml)
