# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import os
import tarfile

import anise.framework.files
import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="name" v="myprojectname" />
    <val k="base.version" v="1.2" />
    <val k="base.summary" v="mysummary" />
    <val k="base.email" v="my@mail.address" />
    <val k="base.authors"><val v="John Doe" /></val>
    <val k="base.license" oref="licensing.PublicDomain" />
    <val k="base.maturity" oref="base.Maturity.ProductionStable" />
    <let k="base.set_basecolor">
        <val k="r" t="float" v="0.1" />
        <val k="g" t="float" v="0.1" />
        <val k="b" t="float" v="0.1" />
    </let>
    <val k="base.longdescription" v="mylongdescription" />
    <let k="documentation.has_developerdocumentation"/>
    <let k="distributables.pool.addgroup" name="source tarball">
        <val k="sources" t="files.TaskExecution">
            <val k="task" oref="packages.tarpackage" />
            <val k="source" t="files.TaskExecution">
                <val k="task" oref="packages.makerawpackage" />
            </val>
        </val>
    </let>
    <let k="releasing.tasks.add" destination="{0}">
        <val k="task" oref="filetransfer.localupload" />
        <val k="source" t="files.TaskExecution">
            <val k="task" oref="homepage.makehomepage" />
        </val>
    </let>
</aniseprojectdesc>
"""

_myfeature = """
import os

import anise.framework.features
import anise.framework.files

from anise.framework.projects import universe


@anise.framework.features.hookhandler(anise.framework.features.HOOK_BEFORE_DEFINITION)
def _prepare_fallbacks():
    universe.versioncontrol.current = MyVCS()

versioncontrol = anise.framework.features.loadfeature("versioncontrol").featuremodule

class MyVCS(versioncontrol.Internals.VersionControlSystem):

    def __init__(self):
        versioncontrol.Internals.VersionControlSystem.__init__(self)

    def getfullversion(self):
        return universe.base.version + "." + universe.revno

    def createrawpackage(self):
        r = anise.framework.files.Filestructure(universe.specialpaths.projectdir).with_modified_rootname("rawpackage")
        os.unlink(r.path + "/y")
        return r

    def syncversioncontrolsystem(self, commitmessage="", forceskipped=True, forceunchanged=False):
        universe.synccnt[0] += 1
        
    def getchangelog(*a, **b):
        return ""
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        self.destination = anise.framework.files.TempDir().path
        super().__init__(*a, projectdesc=_projectdesc.format(self.destination), **kwa)

    def test_a(self):
        anise.utils.basic.writetofile(self.mydir + "/x", "aa")
        anise.utils.basic.writetofile(self.mydir + "/y", "bb")
        anise.utils.basic.mkdirs(self.rootdir + "/features")
        anise.utils.basic.writetofile(self.rootdir + "/features/myfeature.py", _myfeature)
        loadfeaturesfrom = ["builtin", self.rootdir + "/features"]
        synccnt = [0]
        self.runanise("releasing.release", values={'synccnt': synccnt, 'revno': '1337'},
                      loadfeaturesfrom=loadfeaturesfrom)
        p = self.destination
        self.assertGreater(synccnt[0], 0)
        self.assertTrue(os.path.exists(p + "/myprojectname/myprojectname-1.2.1337.tgz"))

        tar = tarfile.open(p + "/myprojectname/myprojectname-1.2.1337.tgz", "r:gz")
        tarcnt = {}
        for tarinfo in tar:
            tarcnt[tarinfo.name] = tarinfo.size
        tar.close()
        self.assertTrue("myprojectname-1.2.1337/x" in tarcnt)
        self.assertFalse("myprojectname-1.2.1337/y" in tarcnt)
