# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.framework.files
import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="name" v="myprojectname" />
    <val k="base.version" v="1.2" />
    <val k="base.summary" v="mysummary" />
    <val k="base.email" v="my@mail.address" />
    <val k="base.authors"><val v="John Doe" /></val>
    <val k="base.license" oref="licensing.PublicDomain" />
    <val k="base.maturity" oref="base.Maturity.ProductionStable" />
    <val k="base.longdescription" v="mylongdescription" />
</aniseprojectdesc>
"""

_myfeature = """
import anise.framework.features

from anise.framework.projects import universe

milieus = anise.framework.features.loadfeature("milieus").featuremodule

tst = ""


class MyMilieu(milieus.Milieu):

    def __init__(self, itxt="I"):
        milieus.Milieu.__init__(self)
        self.itxt = itxt

    def initialize(self):
        global tst
        super().initialize()
        tst += self.itxt

    def call(self, *args, inside_sandbox=True, **kwargs):
        global tst
        tst += args[0]
        

class MyMilieuComposite(milieus.MilieuComposite):

    def __init__(self, ctxt="0"):
        milieus.MilieuComposite.__init__(self)
        self.ctxt = ctxt

    def initialize(self):
        global tst
        super().initialize()
        tst += "J"

    def call(self, *args, inside_sandbox=True, **kwargs):
        global tst
        tst += self.ctxt + args[0]


class MyMilieuComposite2(milieus.MilieuComposite):

    def boo(self, t):
        global tst
        tst += t.upper()


def foo0():
    global tst
    milieu = universe.milieus.get(MyMilieu)
    milieu.createtempdir().path
    milieu.call("a")
    return tst

def foo1():
    global tst
    universe.milieus.get("myfeature.MyMilieu").call("b")
    return tst

def foo2():
    global tst
    universe.milieus.get({"class":MyMilieu}).call("c")
    return tst

def foo3():
    global tst
    universe.milieus.get({"class":MyMilieu, "itxt": "X"}).call("d")
    return tst 
       
def foo4():
    global tst
    universe.milieus.get([{"class":MyMilieu, "itxt": "X"}, 
                                {"class":MyMilieu}]).call("e")
    return tst
    
def foo5():
    global tst
    universe.milieus.get(universe.milieus.get(MyMilieu)).call("f")
    return tst  
       
def foo6():
    global tst
    universe.milieus.get([{"class":MyMilieu}, 
                                {"class":MyMilieuComposite}]).call("g")
    return tst   
       
def foo7():
    global tst
    universe.milieus.get([{"class":MyMilieu, "itxt": "A"}, 
                                {"class":MyMilieuComposite},
                                {"class":MyMilieu, "itxt": "B"}, 
                                {"class":MyMilieuComposite, "ctxt": "1"}]).call("h")
    return tst     
       
def foo8():
    global tst
    universe.milieus.defaultmilieu = universe.milieus.get([
                                {"class":MyMilieu, "itxt": "A"}, 
                                {"class":MyMilieuComposite2},
                                {"class":MyMilieuComposite, "ctxt": "1"}])
    b = universe.milieus.get([{"class":MyMilieuComposite}])
    b.boo("y")
    b.call("i")
    return tst         
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        self.destination = anise.framework.files.TempDir().path
        super().__init__(*a, projectdesc=_projectdesc.format(self.destination), **kwa)

    def test_a(self):
        anise.utils.basic.mkdirs(self.rootdir + "/features")
        anise.utils.basic.writetofile(self.rootdir + "/features/myfeature.py", _myfeature)
        loadfeaturesfrom = ["builtin", self.rootdir + "/features"]
        for i, res in enumerate(["Ia", "Ib", "Ic", "Xd", "Xe", "If", "IJ0g", "AJJ1h", "AJJY0i"]):
            self.assertEqual(self.runanise("myfeature.foo"+str(i), loadfeaturesfrom=loadfeaturesfrom), res)
